package com.bomostory.sceneeditmodule

import android.content.Context
import android.graphics.*
import com.example.tfat.myapplication.R

object CoverDrawer {
    private const val DEFAULT_HEIGHT = 1080f

    fun drawFrontCover(width: Int, height: Int, name: String, coverColor: Int): Bitmap {
        val scale = height.toFloat() / DEFAULT_HEIGHT
        val paint = Paint(Paint.ANTI_ALIAS_FLAG).apply {
            color = Color.WHITE
            textSize = 100f * scale
            textAlign = Paint.Align.CENTER
            flags += Paint.FAKE_BOLD_TEXT_FLAG
        }
        return Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_4444).apply {
            Canvas(this).apply {
                drawColor(coverColor)
                val marginBottom = height / 20
                val rect = Rect(0, height - height / 3 - marginBottom, width, height - marginBottom)
                val p = Paint(Paint.ANTI_ALIAS_FLAG).apply {
                    color = Color.parseColor("#66ffffff")
                    style = Paint.Style.FILL_AND_STROKE
                }
                drawRect(rect, p)
                drawText(name, rect.centerX().toFloat(), rect.centerY().toFloat(), paint)
            }
        }
    }

    fun drawBackCover(context: Context, width: Int, height: Int, coverColor: Int): Bitmap {
        return Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_4444).apply {
            Canvas(this).apply {
                drawColor(coverColor)
                val drawable = context.getDrawable(R.drawable.ic_logo_full_white)
                val marginBottom = this.height / 10
                val w = this.width / 3
                val scale = w.toFloat() / drawable.intrinsicWidth.toFloat()
                val h = drawable.intrinsicHeight * scale
                val left = (this.width - w) / 2
                val top = (this.height - h) - marginBottom
                val right = (this.width + w) / 2
                val bottom = (this.height) - marginBottom
                drawable.setBounds(left, top.toInt(), right, bottom)
                drawable.draw(this)
            }
        }
    }
}