package com.bomostory.sceneeditmodule

import android.content.Context
import android.graphics.*
import android.graphics.drawable.BitmapDrawable
import android.graphics.drawable.VectorDrawable
import android.support.v4.content.ContextCompat
import com.bomostory.sceneeditmodule.basicdata.Actor
import com.example.tfat.myapplication.R
import android.graphics.Bitmap



object DialogueDrawer {
    fun drawDialogue(context:Context, data: Actor): Bitmap? {
        val conf = Bitmap.Config.ARGB_8888
        var dialogBitmap = Bitmap.createBitmap(data.sideLength, data.sideHeight, conf)
        dialogBitmap = Bitmap.createScaledBitmap(dialogBitmap, data.sideLength, data.sideHeight, true)
        val canvas = Canvas(dialogBitmap)
        val textPaint = Paint()
        textPaint.strokeWidth = 1f
        textPaint.isAntiAlias = true
        textPaint.color = Color.parseColor(data.textColor)
        textPaint.textSize = 40f
        textPaint.textAlign = data.textAlign
        var mResId = data.resourcePath.toInt()
        val drawable = ContextCompat.getDrawable(context, mResId)
        var valuePaddingInPixels = context.resources.getDimension(R.dimen.edit_actor_padding)
        if (drawable is BitmapDrawable) {
            var bitmap = BitmapFactory.decodeResource(context.resources, mResId)
            canvas?.drawBitmap(bitmap,0f,0f, null)
        } else if (drawable is VectorDrawable) {
            var bitmap = getBitmap(data,drawable)
            if(data.isMirror) {
                val matrix = Matrix()
                matrix.postScale(-1f, 1f)
                val new2 = Bitmap.createBitmap(bitmap, 0, 0, bitmap.width, bitmap.height, matrix, true)
                canvas?.drawBitmap(new2, null, RectF(valuePaddingInPixels, valuePaddingInPixels,
                        data.sideLength.toFloat() - valuePaddingInPixels,
                        data.sideHeight.toFloat() - valuePaddingInPixels), null)
            } else {
                canvas?.drawBitmap(bitmap, null, RectF(valuePaddingInPixels, valuePaddingInPixels,
                        data.sideLength.toFloat() - valuePaddingInPixels,
                        data.sideHeight.toFloat() - valuePaddingInPixels), null)
            }
        } else {
            throw IllegalArgumentException("unsupported drawable type")
        }
        val textPositionX: Float
        val textPositionY: Float
        when(data.textAlign){
            Paint.Align.LEFT -> {
                textPositionX = context.resources.getDimension(R.dimen.dialogue_padding)
                textPositionY = data.sideHeight.div(2).toFloat()
            }
            Paint.Align.CENTER -> {
                textPositionX = data.sideLength.div(2).toFloat()
                textPositionY = data.sideHeight.div(2).toFloat()
            }
            Paint.Align.RIGHT -> {
                textPositionX = data.sideLength - context.resources.getDimension(R.dimen.dialogue_padding)
                textPositionY = data.sideHeight.div(2).toFloat()
            }
        }
        canvas?.drawText(data.text, textPositionX,textPositionY, textPaint)
    return dialogBitmap
    }

    private fun getBitmap(data: Actor, vectorDrawable: VectorDrawable): Bitmap {
        val bitmap = Bitmap.createBitmap(data.sideLength,
                data.sideHeight, Bitmap.Config.ARGB_8888)
        val canvas = Canvas(bitmap)
        vectorDrawable.setBounds(0, 0, canvas.width, canvas.height)
        vectorDrawable.draw(canvas)
        return bitmap
    }
}