package com.bomostory.sceneeditmodule

import android.content.Context
import com.bomostory.pdfexport.PdfBookWriter
import com.bomostory.pdfexport.StandardPdfWriter
import com.bomostory.sceneeditmodule.basicdata.Project
import io.reactivex.Observable
import java.io.File

object PdfMaker {
    fun makeStandard(context: Context, project: Project, file: File): Observable<Float> {
        return Observable.create { emitter ->
            val total = (2 + project.story?.scenes?.size!!).toFloat()
            var progress = 0
            val width = 1920
            val height = 1080
            val coverWidth = width / 2
            emitter.onNext(progress++ / total)
            val front = CoverDrawer.drawFrontCover(coverWidth, height, project.name
                    ?: "", project.frontCoverColor.getColor(context))
            emitter.onNext(progress++ / total)
            val back = CoverDrawer.drawBackCover(context, coverWidth, height, project.backCoverColor.getColor(context))
            emitter.onNext(progress++ / total)
            val standardPdfWriter = StandardPdfWriter(file, front, back)
            project.story?.scenes?.forEach {
                val bitmap = SceneDrawer.drawScene(context, it, 0, width, height) ?: return@forEach
                standardPdfWriter.addImage(bitmap)
                emitter.onNext(progress++ / total)
            }
            standardPdfWriter.finish()
            emitter.onComplete()
        }
    }

    fun makeBooklet(context: Context, project: Project, file: File): Observable<Float> {
        return Observable.create { emitter ->
            val total = (2 + project.story?.scenes?.size!!).toFloat()
            var progress = 0
            val width = 1920 / 2
            val height = 1080
            val name = project?.name ?: ""
            val author = project?.author ?: ""
            emitter.onNext(progress++ / total)
            /** front **/
            val front = CoverDrawer.drawFrontCover(width, height, name, project.frontCoverColor.getColor(context))
            emitter.onNext(progress++ / total)
            /** back **/
            val back = CoverDrawer.drawBackCover(context, width, height, project.backCoverColor.getColor(context))
            emitter.onNext(progress++ / total)
            /** pages **/
            val pdfBookWriter = PdfBookWriter(file, front, back)
            project.story?.scenes?.forEach {
                val bitmap = SceneDrawer.drawScene(context, it, 0, width, height) ?: return@forEach
                pdfBookWriter.addImage(bitmap)
                emitter.onNext(progress++ / total)
            }
            pdfBookWriter.finish()
            emitter.onComplete()
        }
    }

}