package com.bomostory.sceneeditmodule

import android.content.Context
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Canvas
import android.graphics.Paint
import android.graphics.drawable.VectorDrawable
import com.bomostory.sceneeditmodule.basicdata.Scene
import kotlin.math.pow

object SceneDrawer {
    fun drawScene(context: Context, scene: Scene, trackX: Int, scaleWidth: Int, scaleHeight: Int): Bitmap? {
        var sceneBitmap: Bitmap? = null

        scene?.apply {
            sceneBitmap = BitmapFactory.decodeFile(backgroundPath)
            sceneBitmap = Bitmap.createScaledBitmap(sceneBitmap, scaleWidth, scaleHeight, true)

            val canvas = Canvas(sceneBitmap)
            for (layer in layers) {
                for (actor in layer.actors) {
                    if (!actor.isDialogue) {
                        var actorBitmap = BitmapFactory.decodeFile(actor.resourcePath)
                        actorBitmap = Bitmap.createScaledBitmap(actorBitmap, actor.sideLength, actor.sideHeight, true)

                        canvas.save()
                        canvas.translate(trackX / 2f.pow(layers.indexOf(layer)), 0f)
                        canvas.drawBitmap(actorBitmap, actor.positionX.toFloat(), actor.positionY.toFloat(), Paint())
                        canvas.restore()

                        if (!actorBitmap.isRecycled) {
                            actorBitmap.recycle()
                        }
                    } else {
                        canvas.save()
                        canvas.translate(trackX / 2f.pow(layers.indexOf(layer)), 0f)
                        canvas?.drawBitmap(DialogueDrawer.drawDialogue(context, actor), actor.positionX.toFloat(), actor.positionY.toFloat(), null)
                        canvas.restore()
                    }

                }
            }
        }
        return sceneBitmap
    }
}