package com.bomostory.sceneeditmodule.cover

import android.content.Context
import android.support.constraint.ConstraintLayout
import android.util.AttributeSet
import android.view.View
import com.example.tfat.myapplication.R

class CoverColorSelectorView @JvmOverloads constructor(
        context: Context,
        attrs: AttributeSet? = null,
        defStyleAttr: Int = 0
) : ConstraintLayout(context, attrs, defStyleAttr) {

    private val viewToColorMap = HashMap<Int, CoverColor>().apply {
        put(R.id.iv_coverColorSelector_color1, CoverColor.Color1)
        put(R.id.iv_coverColorSelector_color2, CoverColor.Color2)
        put(R.id.iv_coverColorSelector_color3, CoverColor.Color3)
        put(R.id.iv_coverColorSelector_color4, CoverColor.Color4)
        put(R.id.iv_coverColorSelector_color5, CoverColor.Color5)
        put(R.id.iv_coverColorSelector_color6, CoverColor.Color6)
        put(R.id.iv_coverColorSelector_color7, CoverColor.Color7)
        put(R.id.iv_coverColorSelector_color8, CoverColor.Color8)
        put(R.id.iv_coverColorSelector_color9, CoverColor.Color9)
        put(R.id.iv_coverColorSelector_color10, CoverColor.Color10)
    }

    private val colorToViewMap = HashMap<CoverColor, Int>().apply {
        put(CoverColor.Color1, R.id.iv_coverColorSelector_color1)
        put(CoverColor.Color2, R.id.iv_coverColorSelector_color2)
        put(CoverColor.Color3, R.id.iv_coverColorSelector_color3)
        put(CoverColor.Color4, R.id.iv_coverColorSelector_color4)
        put(CoverColor.Color5, R.id.iv_coverColorSelector_color5)
        put(CoverColor.Color6, R.id.iv_coverColorSelector_color6)
        put(CoverColor.Color7, R.id.iv_coverColorSelector_color7)
        put(CoverColor.Color8, R.id.iv_coverColorSelector_color8)
        put(CoverColor.Color9, R.id.iv_coverColorSelector_color9)
        put(CoverColor.Color10, R.id.iv_coverColorSelector_color10)
    }

    private val colorViewList = ArrayList<View>()

    init {
        inflate(context, R.layout.view_cover_color_selector, this)
        viewToColorMap.keys.forEach { ivId ->
            val v = findViewById<View>(ivId)
            v.setOnClickListener { it ->
                viewToColorMap[it.id]?.let { color -> selectedColor = color }
            }
            colorViewList.add(v)
        }
    }

    var selectedColor: CoverColor = CoverColor.Color1
        set(value) {
            field = value
            select(value)
            onChangeListener?.onChange()
        }
    var onChangeListener: OnChangeListener? = null

    private fun select(coverColor: CoverColor) {
        colorToViewMap[coverColor]?.let {
            val v = findViewById<View>(it)
            colorViewList.forEach {
                if (it == v) {
                    it.setToCheckState()
                } else {
                    it.setToNonCheckState()
                }
            }
        }
    }

    private fun View.setToNonCheckState() {
        setPadding(0)
    }

    private fun View.setToCheckState() {
        val padding = resources.getDimension(R.dimen.cover_color_item_padding).toInt()
        setPadding(padding)
    }

    private fun View.setPadding(padding: Int) {
        setPadding(padding, padding, padding, padding)
    }

    interface OnChangeListener {
        fun onChange()
    }
}