package com.bomostory.sceneeditmodule.cover

import android.os.Bundle
import android.support.design.widget.TabLayout
import android.support.v4.app.DialogFragment
import android.support.v4.app.FragmentManager
import android.support.v4.view.PagerAdapter
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.example.tfat.myapplication.R
import kotlinx.android.synthetic.main.dialog_cover_editor.*
import kotlinx.android.synthetic.main.dialog_cover_editor.view.*

class CoverEditorDialog : DialogFragment() {
    companion object {
        const val FRONT_PAGE_POSITION = 0
        const val PAGE_COUNT = 2
    }

    init {
        isCancelable = false
    }

    private val dialogTag = this::class.java.simpleName

    lateinit var onViewCreated: Runnable
    var onSave: View.OnClickListener? = null
        set(value) {
            field = value
            frontCoverEditorView_coverEditorDialog.onSave = value
            backCoverEditorView_coverEditorDialog.onSave = value
        }
    var onCancel: View.OnClickListener? = null
        set(value) {
            field = value
            frontCoverEditorView_coverEditorDialog.onCancel = value
            backCoverEditorView_coverEditorDialog.onCancel = value
        }
    var storyName: String
        get () {
            return frontCoverEditorView_coverEditorDialog.storyName
        }
        set(value) {
            frontCoverEditorView_coverEditorDialog.storyName = value
        }
    var author: String
        get() {
            return frontCoverEditorView_coverEditorDialog.author
        }
        set(value) {
            frontCoverEditorView_coverEditorDialog.author = value
        }
    var frontCoverColor: CoverColor
        get() {
            return frontCoverEditorView_coverEditorDialog.selectedColor
        }
        set(value) {
            frontCoverEditorView_coverEditorDialog.selectedColor = value
        }
    var backCoverColor: CoverColor
        get() {
            return backCoverEditorView_coverEditorDialog.selectedColor
        }
        set(value) {
            backCoverEditorView_coverEditorDialog.selectedColor = value
        }
    var category: Category
        get() {
            return frontCoverEditorView_coverEditorDialog.category
        }
        set(value) {
            frontCoverEditorView_coverEditorDialog.category = value
        }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.dialog_cover_editor, container, false).apply {
            val viewPager = viewPager_coverEditorDialog_content
            val tabLayout = tabLayout_coverEditorDialog_header
            viewPager.adapter = object : PagerAdapter() {
                override fun instantiateItem(container: ViewGroup, position: Int): Any {
                    return container.getChildAt(position)
                }

                override fun isViewFromObject(p0: View, p1: Any): Boolean {
                    return p0 == p1
                }

                override fun getCount(): Int {
                    return PAGE_COUNT
                }

                override fun destroyItem(container: ViewGroup, position: Int, `object`: Any) {
                    // no super
                }
            }
            viewPager.addOnPageChangeListener(TabLayout.TabLayoutOnPageChangeListener(tabLayout))
            tabLayout.apply {
                addTab(newTab().setText(R.string.cover_editor_dialog_tab_front))
                addTab(newTab().setText(R.string.cover_editor_dialog_tab_back))
                addOnTabSelectedListener(object : TabLayout.OnTabSelectedListener {
                    override fun onTabReselected(tab: TabLayout.Tab?) {}
                    override fun onTabUnselected(tab: TabLayout.Tab?) {}
                    override fun onTabSelected(tab: TabLayout.Tab?) {
                        tab?.let { viewPager.currentItem = it.position }
                    }
                })
            }
            frontCoverEditorView_coverEditorDialog.apply {
                this.onSave = this@CoverEditorDialog.onSave
                this.onCancel = this@CoverEditorDialog.onCancel
            }
            backCoverEditorView_coverEditorDialog.apply {
                this.onSave = this@CoverEditorDialog.onSave
                this.onCancel = this@CoverEditorDialog.onCancel
            }
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        onViewCreated.run()
    }

    fun show(manager: FragmentManager?) {
        manager?.apply {
            val ft = beginTransaction()
            findFragmentByTag(dialogTag)?.let { ft.remove(it) }
            ft.addToBackStack(null)
            ft.commit()
        }
        super.show(manager, dialogTag)
    }

    fun showDuplicatedProjectNameMsg() {
        viewPager_coverEditorDialog_content.currentItem = FRONT_PAGE_POSITION
        frontCoverEditorView_coverEditorDialog.showDuplicatedProjectNameMsg()
    }

    fun showEmptyProjectNameMsg() {
        viewPager_coverEditorDialog_content.currentItem = FRONT_PAGE_POSITION
        frontCoverEditorView_coverEditorDialog.showEmptyProjectNameMsg()
    }
}