package com.bomostory.sceneeditmodule.cover

import android.content.Context
import android.text.Editable
import android.text.TextWatcher
import android.util.AttributeSet
import android.view.View
import android.widget.ArrayAdapter
import com.example.tfat.myapplication.R
import kotlinx.android.synthetic.main.view_front_cover_editor.view.*

class FrontCoverEditorView @JvmOverloads constructor(
        context: Context,
        attrs: AttributeSet? = null,
        defStyleAttr: Int = 0
) : BaseCoverEditorView(context, attrs, defStyleAttr) {

    var onCancel: View.OnClickListener? = null
    var onSave: View.OnClickListener? = null
    var selectedColor: CoverColor
        get() {
            return coverColorSelectorView_frontCoverEditor.selectedColor
        }
        set(value) {
            coverColorSelectorView_frontCoverEditor.selectedColor = value
        }
    var storyName: String
        get() {
            return et_frontCoverEditor_projectName.text.toString()
        }
        set(value) {
            et_frontCoverEditor_projectName.setText(value)
        }
    var author: String
        get() {
            return et_frontCoverEditor_author.text.toString()
        }
        set(value) {
            et_frontCoverEditor_author.setText(value)
        }
    var category: Category
        get() {
            val position = spinner_frontCoverEditor_category.selectedItemPosition
            return Category.values()[position]
        }
        set(value) {
            val position = value.ordinal
            spinner_frontCoverEditor_category.setSelection(position)
        }

    init {
        View.inflate(context, R.layout.view_front_cover_editor, this)
        coverColorSelectorView_frontCoverEditor.apply {
            onChangeListener = object : CoverColorSelectorView.OnChangeListener {
                override fun onChange() {
                    setCoverColor(selectedColor, this@FrontCoverEditorView.iv_frontCoverEditor_cover)
                }
            }
        }
        btn_frontCoverEditor_save.setOnClickListener { onSave?.onClick(it) }
        btn_frontCoverEditor_cancel.setOnClickListener { onCancel?.onClick(it) }

        // category spinner
        val itemLayoutResId = android.R.layout.simple_dropdown_item_1line
        spinner_frontCoverEditor_category.adapter = ArrayAdapter<String>(context, itemLayoutResId).apply {
            Category.values().forEach {
                val s = context.getString(it.stringResId)
                add(s)
            }
        }

        et_frontCoverEditor_projectName.addTextChangedListener(object : TextWatcher {
            override fun afterTextChanged(editable: Editable?) {
                if (editable == null) return
                tv_frontCoverEditor_coverTitle.text = editable.toString()
            }

            override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
                // do nothing
            }

            override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
                // do nothing
            }
        })
        et_frontCoverEditor_author.addTextChangedListener(object : TextWatcher {
            override fun afterTextChanged(editable: Editable?) {
                if (editable == null) return
                tv_frontCoverEditor_coverAuthor.text = editable.toString()
            }

            override fun beforeTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
                // do nothing
            }

            override fun onTextChanged(p0: CharSequence?, p1: Int, p2: Int, p3: Int) {
                // do nothing
            }
        })
    }

    fun showDuplicatedProjectNameMsg() {
        val msg = context.getString(R.string.front_cover_editor_dialog_duplicated_story_name_msg)
        et_frontCoverEditor_projectName.error = msg
    }

    fun showEmptyProjectNameMsg() {
        val msg = context.getString(R.string.front_cover_editor_dialog_empty_story_name_msg)
        et_frontCoverEditor_projectName.error = msg
    }
}