package com.bomostory.sceneeditmodule.screen.movie.music

import android.content.Context
import android.content.DialogInterface
import android.os.Bundle
import android.support.v4.app.DialogFragment
import android.support.v4.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.SeekBar
import com.bomostory.sceneeditmodule.basicdata.Music
import com.bomostory.sceneeditmodule.utils.TimeUtils
import com.example.audiotrack.EditAudioTrackView

import com.example.tfat.myapplication.R
import com.google.gson.Gson
import kotlinx.android.synthetic.main.fragment_music_edit_dialog.*
import kotlinx.android.synthetic.main.fragment_music_edit_dialog.view.*

private const val ARG_PARAM1 = "editMusic"

/**
 * A simple [Fragment] subclass.
 * Activities that contain this fragment must implement the
 * [MusicEditDialog.OnFragmentInteractionListener] interface
 * to handle interaction events.
 * Use the [MusicEditDialog.newInstance] factory method to
 * create an instance of this fragment.
 *
 */
class MusicEditDialog : DialogFragment() {
    private var editMusic: Music? = null
    private var listener: OnFragmentInteractionListener? = null
    private var dismissListener: DialogInterface.OnDismissListener? = null

    var currentPosition: Int = 0
        set(value) {
            field = value

            editMusic?.let {
                val endPosition = endScrollRatio * it.duration

                if (field >= endPosition) {
                    field = (headScrollRatio * it.duration).toInt()

                    listener?.onMusicStop()

                    editAudioTrackView.playProgressDialogVisible = false
                    play.visibility = View.VISIBLE
                    pause.visibility = View.INVISIBLE
                }
                editAudioTrackView.playProgress = field.toFloat() / it.duration
                editAudioTrackView.playProgressText = TimeUtils.getAudioTimeFormat(field.toLong())
            }
        }

    var headScrollRatio: Double = 0.0
        set(value) {
            field = value

            updateMusicLength()
            updateEditAudioTrackHeadText()
        }

    var endScrollRatio: Double = 1.0
        set(value) {
            field = value

            updateMusicLength()
            updateEditAudioTrackEndText()
        }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        arguments?.let {
            editMusic = Gson().fromJson(it.getString(ARG_PARAM1), Music::class.java)
        }
    }

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?,
                              savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.fragment_music_edit_dialog, container, false).apply {
            editMusic?.let {
                musicName.text = it.name
                musicLength.text = getString(R.string.music_length, TimeUtils.getPlayMovieTimeFormat((it.endTime - it.startTime)))
                startTime.text = TimeUtils.getPlayMovieTimeFormat(0)
                endTime.text = TimeUtils.getPlayMovieTimeFormat(it.endTime)
                volumeSeekBar.progress = (it.volume * volumeSeekBar.max).toInt()
                volumeText.text = "${volumeSeekBar.progress} %"

                play.setOnClickListener { _ ->
                    val startPosition = it.duration * headScrollRatio
                    var playPosition = currentPosition

                    if (startPosition > currentPosition) {
                        playPosition = startPosition.toInt()
                    }

                    listener?.onMusicPlayClick(it, playPosition)

                    editAudioTrackView.playProgressDialogVisible = true
                    play.visibility = View.INVISIBLE
                    pause.visibility = View.VISIBLE
                }

                pause.setOnClickListener { _ ->
                    listener?.onMusicPauseClick()

                    editAudioTrackView.playProgressDialogVisible = false
                    play.visibility = View.VISIBLE
                    pause.visibility = View.INVISIBLE
                }

                volumeSeekBar.setOnSeekBarChangeListener(object : SeekBar.OnSeekBarChangeListener {
                    override fun onProgressChanged(p0: SeekBar?, p1: Int, p2: Boolean) {
                        volumeText.text = "$p1 %"

                        val volume = p1.toFloat() / volumeSeekBar.max.toFloat()
                        listener?.onMusicVolumeChanged(volume)
                    }

                    override fun onStartTrackingTouch(p0: SeekBar?) {
                    }

                    override fun onStopTrackingTouch(p0: SeekBar?) {
                    }
                })

                editAudioTrackView.onHeadScrollListener = object : EditAudioTrackView.OnEditScrollListener {
                    override fun onScroll(view: EditAudioTrackView, scrollRatio: Double) {
                        headScrollRatio = scrollRatio
                    }
                }

                editAudioTrackView.onEndScrollListener = object : EditAudioTrackView.OnEditScrollListener {
                    override fun onScroll(view: EditAudioTrackView, scrollRatio: Double) {
                        endScrollRatio = scrollRatio
                    }

                }

                cancel.setOnClickListener { _ ->
                    dismiss()
                }

                save.setOnClickListener { _ ->
                    editMusic?.apply {
                        startTime = (headScrollRatio * duration).toLong()
                        endTime = (endScrollRatio * duration).toLong()
                        duration = endTime - startTime
                        volume = volumeSeekBar.progress.toFloat() / volumeSeekBar.max.toFloat()

                        listener?.onMusicEditFinish(this)
                    }
                    dismiss()
                }
            }
        }
    }

    private fun updateEditAudioTrackHeadText() {
        editAudioTrackView?.let {
            editMusic?.apply {
                val newDuration = duration * headScrollRatio
                it.headScrollProgressText = TimeUtils.getAudioTimeFormat(newDuration.toLong())
            }
        }
    }

    private fun updateEditAudioTrackEndText() {
        editAudioTrackView?.let {
            editMusic?.apply {
                val newDuration = duration * endScrollRatio
                it.endScrollProgressText = TimeUtils.getAudioTimeFormat(newDuration.toLong())
            }
        }
    }

    private fun updateMusicLength() {
        musicLength?.let {
            editMusic?.apply {
                val newDuration = duration * (-headScrollRatio + endScrollRatio)
                musicLength.text = getString(R.string.music_length, TimeUtils.getPlayMovieTimeFormat(newDuration.toLong()))
            }
        }
    }

    override fun onAttach(context: Context) {
        super.onAttach(context)
        if (context is OnFragmentInteractionListener) {
            listener = context
        } else {
            throw RuntimeException(context.toString() + " must implement OnFragmentInteractionListener")
        }

        if (context is DialogInterface.OnDismissListener) {
            dismissListener = context
        }
    }

    override fun onDetach() {
        super.onDetach()
        listener = null
        dismissListener = null
    }

    override fun onDismiss(dialog: DialogInterface?) {
        super.onDismiss(dialog)
        dismissListener?.onDismiss(dialog)
    }

    /**
     * This interface must be implemented by activities that contain this
     * fragment to allow an interaction in this fragment to be communicated
     * to the activity and potentially other fragments contained in that
     * activity.
     *
     *
     * See the Android Training lesson [Communicating with Other Fragments]
     * (http://developer.android.com/training/basics/fragments/communicating.html)
     * for more information.
     */
    interface OnFragmentInteractionListener {
        fun onMusicPlayClick(music: Music, startPosition: Int)
        fun onMusicPauseClick()
        fun onMusicVolumeChanged(volume: Float)
        fun onMusicStop()
        fun onMusicEditFinish(music: Music)
    }

    companion object {
        @JvmStatic
        fun newInstance(editMusic: String) =
                MusicEditDialog().apply {
                    arguments = Bundle().apply {
                        putString(ARG_PARAM1, editMusic)
                    }
                }
    }
}
