package com.bomostory.sceneeditmodule.screen.view

import android.content.Context
import android.graphics.*
import android.util.AttributeSet
import android.view.View
import android.graphics.drawable.VectorDrawable
import android.graphics.Bitmap
import android.view.MotionEvent
import android.widget.RelativeLayout
import com.bomostory.sceneeditmodule.DialogueDrawer
import com.bomostory.sceneeditmodule.basicdata.Actor
import kotlinx.android.synthetic.main.actor_view.view.*

class DialogueView : EditActorView{

    constructor(context: Context) : super(context)

    constructor(context: Context, attrs: AttributeSet) : super(context, attrs)

    constructor(context: Context, attrs: AttributeSet, defStyle: Int) : super(context, attrs, defStyle)

    var dialogue = Actor()
        set(value) {
            value?.let {
                field = value
                val layoutParams = RelativeLayout.LayoutParams(it.sideLength, it.sideHeight)
                layoutParams.leftMargin = it.positionX
                layoutParams.topMargin = it.positionY
                actor_layout.layoutParams = layoutParams
                val actorLayoutParams = RelativeLayout.LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT)
                actor_view_layout.layoutParams = actorLayoutParams
                when(it.isMovable) {
                    true -> actor_view_layout.setOnTouchListener(actorOnTouchListener())
                }
                when(it.isSelect){
                    true -> change_size_view.visibility = View.VISIBLE
                    false -> change_size_view.visibility = View.GONE
                }
            }
        }


    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        canvas?.drawBitmap(DialogueDrawer.drawDialogue(context, dialogue), dialogue.positionX.toFloat(), dialogue.positionY.toFloat(), null)
        if(dialogue.isSelect) {
            val mPaint = Paint()
            mPaint.style = Paint.Style.STROKE
            mPaint.strokeWidth = 5f
            mPaint.isAntiAlias = true
            mPaint.color = Color.parseColor("#4e342e")
            mPaint.pathEffect = DashPathEffect(floatArrayOf(8f, 8f), 0f)
            canvas?.drawLine(dialogue.positionX.toFloat() + 25 , dialogue.positionY.toFloat() + 25, dialogue.positionX.toFloat() + 25, dialogue.positionY.toFloat() + 25 + dialogue.sideHeight , mPaint)
            canvas?.drawLine(dialogue.positionX.toFloat() + 25, dialogue.positionY.toFloat() + 25, dialogue.positionX.toFloat() + dialogue.sideLength.toFloat() - 25, dialogue.positionY.toFloat() + 25, mPaint)
            canvas?.drawLine(dialogue.positionX.toFloat() + 25, dialogue.positionY.toFloat() + dialogue.sideHeight.toFloat() - 25, dialogue.positionX.toFloat()  + dialogue.sideLength.toFloat() + 25, dialogue.positionY.toFloat() + dialogue.sideHeight.toFloat() - 25, mPaint)
            canvas?.drawLine(dialogue.positionX.toFloat() + dialogue.sideLength.toFloat() - 25, dialogue.positionY.toFloat() + 25, dialogue.positionX.toFloat() + dialogue.sideLength.toFloat() - 25, dialogue.positionY.toFloat() + dialogue.sideHeight.toFloat() - 25, mPaint)
        }
    }

    private fun actorOnTouchListener() : View.OnTouchListener {
        var clickPeriod = 0L
        var positionStart = 0
        var positionEnd = 0
        var mx: Int
        var my: Int
        return View.OnTouchListener { view, motionEvent ->
            when (motionEvent.action) {
                MotionEvent.ACTION_DOWN -> {
                    clickPeriod = System.currentTimeMillis()
                    positionStart = (motionEvent.x).toInt()
                    positionEnd = (motionEvent.y).toInt()
                }
                MotionEvent.ACTION_MOVE -> {
                    if (!(Math.abs(motionEvent.x - positionStart) < 10 && Math.abs(motionEvent.y - positionEnd) < 10) && dialogue.isSelect) {
                        mx = motionEvent.rawX.toInt() - positionStart
                        my = motionEvent.rawY.toInt() - positionEnd
                        var laParams = actor_layout.layoutParams as RelativeLayout.LayoutParams
                        laParams.setMargins(mx, my - 200, 0, 0)
                        actor_layout.layoutParams = laParams
                        val actorLayoutParams = RelativeLayout.LayoutParams(LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT)
                        view.layoutParams = actorLayoutParams
                        dialogue.positionX = mx
                        dialogue.positionY = my - 200
                    }
                }
                MotionEvent.ACTION_UP -> {
                    if (Math.abs(motionEvent.x - positionStart) < 10 && Math.abs(motionEvent.y - positionEnd) < 10 && (System.currentTimeMillis() - clickPeriod) > 1000) {
                        actorCallback?.onActorLongClick(dialogue)
                    } else if (Math.abs(motionEvent.x - positionStart) < 10 && Math.abs(motionEvent.y - positionEnd) < 10) {
                        actorCallback?.onActorSelected(dialogue)
                    } else {
                        if (dialogue.isSelect){
                            actorCallback?.onActorChange(dialogue)
                        }
                    }
                }
            }
            return@OnTouchListener true
        }
    }

    override fun rightBottomSizeChangeListener() : View.OnTouchListener {
        var positionX = 0
        var positionY = 0
        var sizeX = 0
        var sizeY = 0
        var x = 0
        var y = 0
        return View.OnTouchListener { view, motionEvent ->
            when (motionEvent.action) {
                MotionEvent.ACTION_DOWN -> {
                    val layoutParams = actor_layout.layoutParams as RelativeLayout.LayoutParams
                    sizeX = layoutParams.width
                    sizeY = layoutParams.height
                    x = layoutParams.leftMargin
                    y = layoutParams.topMargin
                    positionX = (motionEvent.rawX).toInt()
                    positionY = (motionEvent.rawY).toInt()
                }
                MotionEvent.ACTION_MOVE -> {
                    var mx =  motionEvent.rawX.toInt() - positionX
                    var my =  motionEvent.rawY.toInt() - positionY
                    var dP = when (Math.abs(mx) > Math.abs(my)){
                        true ->  my
                        false -> mx
                    }
                    var dialogueData = Actor().apply {
                        resourcePath = dialogue.resourcePath
                        text = dialogue.text
                        textColor = dialogue.textColor
                        textAlign = dialogue.textAlign
                        this.positionX = x
                        this.positionY = y
                        positionZ = dialogue.positionZ
                        sideLength = sizeX + dP
                        sideHeight = sizeY + dP
                        isSelect = dialogue.isSelect
                        parentLayerIndex = dialogue.parentLayerIndex
                        isMovable = dialogue.isMovable
                        isDialogue = dialogue.isDialogue
                        isMirror = dialogue.isMirror
                        opacity = actor.opacity
                    }
                    dialogue = dialogueData
                }
                MotionEvent.ACTION_UP -> {
                    dialogue.isSelect = true
                    actorCallback?.onActorChange(dialogue)
                }
            }
            return@OnTouchListener true
        }
    }

    override fun leftTopSizeChangeListener() : View.OnTouchListener {
        var positionX = 0
        var positionY = 0
        var sizeX = 0
        var sizeY = 0
        var x = 0
        var y = 0
        return View.OnTouchListener { view, motionEvent ->
            when (motionEvent.action) {
                MotionEvent.ACTION_DOWN -> {
                    val layoutParams = actor_layout.layoutParams as RelativeLayout.LayoutParams
                    sizeX = layoutParams.width
                    sizeY = layoutParams.height
                    x = layoutParams.leftMargin
                    y = layoutParams.topMargin
                    positionX = (motionEvent.rawX).toInt()
                    positionY = (motionEvent.rawY).toInt()
                }
                MotionEvent.ACTION_MOVE -> {
                    val mx =  motionEvent.rawX.toInt() - positionX
                    val my =  motionEvent.rawY.toInt() - positionY
                    var dP = when (Math.abs(mx) > Math.abs(my)){
                        true ->  my
                        false -> mx
                    }
                    var dialogueData = Actor().apply {
                        resourcePath = dialogue.resourcePath
                        text = dialogue.text
                        textColor = dialogue.textColor
                        textAlign = dialogue.textAlign
                        this.positionX = x + dP
                        this.positionY = y + dP
                        positionZ = dialogue.positionZ
                        sideLength = sizeX - dP
                        sideHeight = sizeY - dP
                        isSelect = dialogue.isSelect
                        parentLayerIndex = dialogue.parentLayerIndex
                        isMovable = dialogue.isMovable
                        isDialogue = dialogue.isDialogue
                        isMirror = dialogue.isMirror
                        opacity = actor.opacity
                    }
                    dialogue = dialogueData
                }
                MotionEvent.ACTION_UP -> {
                    dialogue.isSelect = true
                    actorCallback?.onActorChange(dialogue)
                }
            }
            return@OnTouchListener true
        }
    }

    override fun rightTopSizeChangeListener() : View.OnTouchListener {
        var positionX = 0
        var positionY = 0
        var sizeX = 0
        var sizeY = 0
        var x = 0
        var y = 0
        return View.OnTouchListener { view, motionEvent ->
            when (motionEvent.action) {
                MotionEvent.ACTION_DOWN -> {
                    val layoutParams = actor_layout.layoutParams as RelativeLayout.LayoutParams
                    sizeX = layoutParams.width
                    sizeY = layoutParams.height
                    x = layoutParams.leftMargin
                    y = layoutParams.topMargin
                    positionX = (motionEvent.rawX).toInt()
                    positionY = (motionEvent.rawY).toInt()
                }
                MotionEvent.ACTION_MOVE -> {
                    var mx =  motionEvent.rawX.toInt() - positionX
                    var my =  motionEvent.rawY.toInt() - positionY
                    var dP = when (Math.abs(mx) > Math.abs(my)){
                        true ->  my
                        false -> mx
                    }
                    var dialogueData = Actor().apply {
                        resourcePath = dialogue.resourcePath
                        text = dialogue.text
                        textColor = dialogue.textColor
                        textAlign = dialogue.textAlign
                        this.positionX = x
                        this.positionY = y + my
                        positionZ = dialogue.positionZ
                        sideLength = sizeX + mx
                        sideHeight = sizeY - my
                        isSelect = dialogue.isSelect
                        parentLayerIndex = dialogue.parentLayerIndex
                        isMovable = dialogue.isMovable
                        isDialogue = dialogue.isDialogue
                        isMirror = dialogue.isMirror
                        opacity = actor.opacity
                    }
                    dialogue = dialogueData
                }
                MotionEvent.ACTION_UP -> {
                    dialogue.isSelect = true
                    actorCallback?.onActorChange(dialogue)
                }
            }
            return@OnTouchListener true
        }
    }

    override fun leftBottomSizeChangeListener() : View.OnTouchListener {
        var positionX = 0
        var positionY = 0
        var sizeX = 0
        var sizeY = 0
        var x = 0
        var y = 0
        return View.OnTouchListener { view, motionEvent ->
            when (motionEvent.action) {
                MotionEvent.ACTION_DOWN -> {
                    val layoutParams = actor_layout.layoutParams as RelativeLayout.LayoutParams
                    sizeX = layoutParams.width
                    sizeY = layoutParams.height
                    x = layoutParams.leftMargin
                    y = layoutParams.topMargin
                    positionX = (motionEvent.rawX).toInt()
                    positionY = (motionEvent.rawY).toInt()
                }
                MotionEvent.ACTION_MOVE -> {
                    var mx =  motionEvent.rawX.toInt() - positionX
                    var my =  motionEvent.rawY.toInt() - positionY
                    var dialogueData = Actor().apply {
                        resourcePath = dialogue.resourcePath
                        text = dialogue.text
                        textColor = dialogue.textColor
                        textAlign = dialogue.textAlign
                        this.positionX = x + mx
                        this.positionY = y
                        positionZ = dialogue.positionZ
                        sideLength = sizeX - mx
                        sideHeight = sizeY + my
                        isSelect = dialogue.isSelect
                        parentLayerIndex = dialogue.parentLayerIndex
                        isMovable = dialogue.isMovable
                        isDialogue = dialogue.isDialogue
                        isMirror = dialogue.isMirror
                        opacity = actor.opacity
                    }
                    dialogue = dialogueData
                }
                MotionEvent.ACTION_UP -> {
                    dialogue.isSelect = true
                    actorCallback?.onActorChange(dialogue)
                }
            }
            return@OnTouchListener true
        }
    }
}