package com.bomostory.sceneeditmodule.screen.view

import android.content.Context
import android.graphics.*
import android.support.constraint.ConstraintLayout
import android.util.AttributeSet
import android.view.MotionEvent
import android.view.View
import com.bomostory.sceneeditmodule.basicdata.Brush
import android.graphics.PorterDuff
import android.graphics.PorterDuffXfermode


class PaintView: ConstraintLayout , View.OnTouchListener{

    constructor(context: Context) : super(context)
    constructor(context: Context, attrs: AttributeSet) : super(context, attrs)
    constructor(context: Context, attrs: AttributeSet, defStyle: Int) : super(context, attrs, defStyle)
    lateinit var bitmap: Bitmap
    private lateinit var bufferCanvas: Canvas
    private var paint = Paint(Paint.ANTI_ALIAS_FLAG or Paint.DITHER_FLAG)
    var brush = Brush()
        set(value) {
            field = value
            paint = Paint(Paint.ANTI_ALIAS_FLAG or Paint.DITHER_FLAG)
            value?.let {
                when(it.brushType){
                    0 -> {
                        paint.style = Paint.Style.STROKE
                        paint.strokeCap = Paint.Cap.ROUND
                        paint.strokeWidth = it.brushSize.toFloat()
                        paint.color = Color.parseColor(brush.brushColor)
                        paint.xfermode = null
                    }
                    4 -> {
                        paint.style = Paint.Style.STROKE
                        paint.strokeCap = Paint.Cap.ROUND
                        paint.strokeWidth = it.brushSize.toFloat()
                        paint.xfermode = PorterDuffXfermode(PorterDuff.Mode.CLEAR)
                    }
                }
            }
        }
    var path = Path()
    private var mX: Float = 0f
    private var mY: Float = 0f
    fun init(){
        bitmap = Bitmap.createBitmap(this.width, this.height, Bitmap.Config.ARGB_8888)
        bitmap.eraseColor(Color.TRANSPARENT)
        bufferCanvas = Canvas(bitmap)
    }
    override fun onTouch(v: View?, event: MotionEvent): Boolean {
        var x = event.x
        var y = event.y
        when(event.action){
            MotionEvent.ACTION_DOWN ->{
                path.reset()
                path.moveTo(x,y)
                mX = x
                mY = y
                invalidate()
            }
            MotionEvent.ACTION_MOVE ->{
                path.lineTo(mX,mY)
                bufferCanvas.drawPath(path, paint)
                mX = x
                mY = y
                invalidate()
            }
            MotionEvent.ACTION_UP -> {
                path.lineTo(mX, mY)
                bufferCanvas.drawPath(path, paint)
                invalidate()
                path.reset()
            }
        }
        return true
    }
    override fun onDraw(canvas: Canvas?) {
        super.onDraw(canvas)
        canvas?.drawBitmap(bitmap,0f,0f, null)
    }
}