package com.bomostory.sceneeditmodule.share

import android.content.Context
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.os.Bundle
import android.support.v4.app.DialogFragment
import android.support.v4.app.FragmentManager
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import com.bomostory.sceneeditmodule.CoverDrawer
import com.bomostory.sceneeditmodule.basicdata.Project
import com.example.tfat.myapplication.R
import kotlinx.android.synthetic.main.dialog_export_pdf.view.*

class ExportPdfDialog : DialogFragment() {
    private val dialogTag = this::class.java.simpleName

    enum class Type {
        Standard,
        Booklet,
    }

    lateinit var project: Project
    var image1Path = ""
    var image2Path = ""
    var image3Path = ""
    var onClickPrint = Runnable { }
    var type = Type.Standard

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.dialog_export_pdf, container, false).apply {
            rb_exportPdfDialog_foldableBooklet.setOnCheckedChangeListener { _, b ->
                if (b) {
                    rb_exportPdfDialog_standard.isChecked = false
                    type = Type.Booklet
                }
            }
            rb_exportPdfDialog_standard.setOnCheckedChangeListener { _, b ->
                if (b) {
                    rb_exportPdfDialog_foldableBooklet.isChecked = false
                    type = Type.Standard
                }
            }
            btn_exportPdfDialog_cancel.setOnClickListener { this@ExportPdfDialog.dismiss() }
            btn_exportPdfDialog_print.setOnClickListener { onClickPrint.run() }
            if (image1Path != "") {
                val bitmap = decodeImage(context, image1Path)
                iv_exportPdfDialog_standard_1.setImageBitmap(bitmap)
                // Picasso not work, don't know why
                /*
                Picasso.get().load(image1Path).into(iv_exportPdfDialog_standard_1)
                */
            }
            if (image2Path != "") {
                val bitmap = decodeImage(context, image2Path)
                iv_exportPdfDialog_standard_2.setImageBitmap(bitmap)
                // Picasso not work, don't know why
                /*
                Picasso.get().load(image2Path).into(iv_exportPdfDialog_standard_2)
                */
            }
            if (image3Path != "") {
                val bitmap = decodeImage(context, image3Path)
                iv_exportPdfDialog_standard_3.setImageBitmap(bitmap)
                // Picasso not work, don't know why
                /*
                Picasso.get().load(image3Path).into(iv_exportPdfDialog_standard_3)
                */
            }
            iv_exportPdfDialog_foldableBookletCoverFront.setImageBitmap(createFrontCover(context))
            iv_exportPdfDialog_foldableBookletCoverBack.setImageBitmap(createBackCover(context))
        }
    }

    private fun createFrontCover(context: Context): Bitmap {
        val width = context.resources.getDimension(R.dimen.export_pdf_cover_width).toInt()
        val height = context.resources.getDimension(R.dimen.export_pdf_cover_height).toInt()
        val name = project.name ?: ""
        val color = project.frontCoverColor.getColor(context)
        return CoverDrawer.drawFrontCover(width, height, name, color)
    }

    private fun createBackCover(context: Context): Bitmap {
        val width = context.resources.getDimension(R.dimen.export_pdf_cover_width).toInt()
        val height = context.resources.getDimension(R.dimen.export_pdf_cover_height).toInt()
        val color = project.backCoverColor.getColor(context)
        return CoverDrawer.drawBackCover(context, width, height, color)
    }

    private fun decodeImage(context: Context, imgPath: String): Bitmap {
        val options = BitmapFactory.Options().apply {
            inJustDecodeBounds = true
        }
        BitmapFactory.decodeFile(imgPath, options)
        options.apply {
            inJustDecodeBounds = false
            val viewWidth = context.resources.getDimension(R.dimen.export_pdf_screenshot_width)
            inSampleSize = (outWidth / viewWidth).toInt()
        }
        return BitmapFactory.decodeFile(imgPath, options)
    }

    fun show(manager: FragmentManager?) {
        manager?.apply {
            val ft = beginTransaction()
            findFragmentByTag(dialogTag)?.let { ft.remove(it) }
            ft.addToBackStack(null)
            ft.commit()
        }
        super.show(manager, dialogTag)
    }
}