package com.bomostory.sceneeditmodule.share

import android.content.DialogInterface
import android.graphics.drawable.Drawable
import android.os.Bundle
import android.support.v4.app.DialogFragment
import android.support.v4.app.FragmentManager
import android.support.v7.widget.LinearLayoutManager
import android.support.v7.widget.RecyclerView
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.SeekBar
import com.bomostory.sceneeditmodule.basicdata.Project
import com.bomostory.sceneeditmodule.basicdata.Scene
import com.bomostory.sceneeditmodule.utils.HorizontalItemDecoration
import com.bomostory.sceneeditmodule.utils.TimeUtils
import com.example.tfat.myapplication.R
import kotlinx.android.synthetic.main.dialog_share.*
import kotlinx.android.synthetic.main.dialog_share.view.*
import kotlinx.android.synthetic.main.movie_item_view.view.*

class ShareDialog : DialogFragment() {

    private val dialogTag = this::class.java.simpleName

     var dismissListener: DialogInterface.OnDismissListener? = null

    var seekBarChangedListener: SeekBar.OnSeekBarChangeListener? = null

    lateinit var project: Project

    var storyPeriod: Long = 0

    var scene: Scene? = null
        set(value) {
            field = value
            movieView_shareDialog_preview?.let {
                it.scene = value
            }
        }

    var x: Int = 0
        set(value) {
            field = value
            movieView_shareDialog_preview?.let {
                it.x = value
            }
        }

    var time: Long = 0
        set(value) {
            field = value
            seekBarMoviePlay?.progress = value.toInt()
            tv_shareDialog_time?.text = "${TimeUtils.getPlayMovieTimeFormat(time)}/${TimeUtils.getPlayMovieTimeFormat(storyPeriod)}"
        }

    var onClickEditCover = Runnable {}
    var onClickExportPdf = Runnable {}
    var onClickPrint = Runnable {}
    var onClickBomo = Runnable {}
    var onClickYoutube = Runnable {}
    var onClickFacebook = Runnable {}
    var onClickTwitter = Runnable {}
    var onClickMore = Runnable {}
    var onClickPlay = Runnable {}


    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {
        return inflater.inflate(R.layout.dialog_share, container, false).apply {
            movieView_shareDialog_preview.scene = project.story?.scenes?.get(0)
            tv_shareDialog_name.text = project.name
            tv_shareDialog_author.text = project.author
            seekBarMoviePlay.max = storyPeriod.toInt()
            tv_shareDialog_time.text = "${TimeUtils.getPlayMovieTimeFormat(time)}/${TimeUtils.getPlayMovieTimeFormat(storyPeriod)}"
            seekBarMoviePlay.setOnSeekBarChangeListener(seekBarChangedListener)
            btn_shareDialog_editCover.setOnClickListener { onClickEditCover.run() }
            btn_shareDialog_exportPdf.setOnClickListener { onClickExportPdf.run() }
            btn_shareDialog_print.setOnClickListener { onClickPrint.run() }
            iv_shareDialog_bomo.setOnClickListener { onClickBomo.run() }
            iv_shareDialog_youtube.setOnClickListener { onClickYoutube.run() }
            iv_shareDialog_facebook.setOnClickListener { onClickFacebook.run() }
            iv_shareDialog_twitter.setOnClickListener { onClickTwitter.run() }
            iv_shareDialog_more.setOnClickListener { onClickMore.run() }
            iv_shareDialog_play.setOnClickListener { onClickPlay.run() }

            sceneList.apply {
                setHasFixedSize(true)
                addItemDecoration(HorizontalItemDecoration(resources.getDimensionPixelSize(R.dimen.horizontal_item_spacing)))
                layoutManager = LinearLayoutManager(context, LinearLayoutManager.HORIZONTAL, false)
                adapter = MovieAdapter()
            }
        }
    }

    fun show(manager: FragmentManager?) {
        manager?.apply {
            val ft = beginTransaction()
            findFragmentByTag(dialogTag)?.let { ft.remove(it) }
            ft.addToBackStack(null)
            ft.commit()
        }
        super.show(manager, dialogTag)
    }

    override fun onDetach() {
        super.onDetach()
        dismissListener = null
    }

    override fun onDismiss(dialog: DialogInterface?) {
        super.onDismiss(dialog)
        dismissListener?.onDismiss(dialog)
    }

    inner class MovieAdapter : RecyclerView.Adapter<MovieAdapter.ViewHolder>() {
        override fun onCreateViewHolder(p0: ViewGroup, p1: Int): ViewHolder {
            val view = LayoutInflater.from(p0.context).inflate(R.layout.scene_item_view, p0, false)
            return ViewHolder(view)
        }

        override fun getItemCount(): Int {
            project.story?.let {
                return it.scenes.size
            }
            return 0
        }

        override fun onBindViewHolder(p0: ViewHolder, p1: Int) {
            p0.bind(p1)
        }

        inner class ViewHolder(view: View) : RecyclerView.ViewHolder(view) {
            fun bind(p1: Int) {
                itemView.apply {
                    project.story?.apply {
                        screenshot.setImageDrawable(Drawable.createFromPath(this.scenes[p1].backgroundPath))
                    }
                }
            }
        }
    }
}