package com.bomostory.sceneeditmodule.utils

import android.content.Context
import android.graphics.Bitmap
import android.net.Uri
import android.provider.MediaStore
import com.bomostory.sceneeditmodule.Config
import com.bomostory.sceneeditmodule.SceneDrawer
import com.bomostory.sceneeditmodule.basicdata.Project
import com.google.gson.Gson
import io.reactivex.Completable
import java.io.BufferedOutputStream
import java.io.File
import java.io.FileOutputStream
import java.io.FileWriter

object FileUtils {

    fun getVideoFile(project: Project): File {
        val projectFolder = File(Config.PROJECTS_FOLDER, project.name)
        return File(projectFolder, Config.VIDEO_FILE_NAME)
    }

    fun saveImageForActor(context: Context, uri: Uri, fileName: String){
        val dir = File(Config.IMAGE_FOLDER,"")
        dir.mkdir()
        val file = File(Config.IMAGE_FOLDER, fileName)
        try {
            val bitmap = MediaStore.Images.Media.getBitmap(context.contentResolver, uri)
            val bos = BufferedOutputStream(FileOutputStream(file))
            bitmap.compress(Bitmap.CompressFormat.JPEG, 80, bos)
            bos.flush()
            bos.close()
            if (context is OnSaveActorImage) {
                context.onSaveActorImage(file.path)
            } else {
                throw RuntimeException(context.toString() + " must implement OnFragmentInteractionListener")
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    fun saveBitmapForActor(context: Context, bitmap: Bitmap?, fileName: String){
        val dir = File(Config.IMAGE_FOLDER,"")
        dir.mkdir()
        val file = File(Config.IMAGE_FOLDER, fileName.plus(".png"))
        try {
            val bos = BufferedOutputStream(FileOutputStream(file))
            bitmap?.compress(Bitmap.CompressFormat.PNG, 80, bos)
            bos.flush()
            bos.close()
            if (context is OnSaveActorImage) {
                context.onSaveActorImage(file.path)
            } else {
                throw RuntimeException(context.toString() + " must implement OnFragmentInteractionListener")
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    fun changeProjectName(oldName: String, newName: String): Boolean {
        val oldProjectFolder = File(Config.PROJECTS_FOLDER, oldName)
        val newProjectFolder = File(Config.PROJECTS_FOLDER, newName)
        return oldProjectFolder.renameTo(newProjectFolder)
    }

    fun saveProject(context: Context, project: Project, scaleWidth: Int, scaleHeight: Int): Completable {
        return Completable.create {
            try {
                val fileWriter = FileWriter(File(File(Config.PROJECTS_FOLDER, project.name),Config.PROJECT_FILE_NAME))
                fileWriter.write(Gson().toJson(project))
                fileWriter.flush()
                when (project.story?.scenes?.size) {
                    1 -> {
                        var fileOutputStream = FileOutputStream(project.scene1File)
                        project.story?.let {
                            it.scenes?.let {
                                SceneDrawer.drawScene(context, it[0], 0, scaleWidth, scaleHeight)?.compress(Bitmap.CompressFormat.PNG,
                                        100, fileOutputStream)
                                fileOutputStream.flush()
                                fileOutputStream.close()
                            }
                        }
                    }
                    else -> {
                        var fileOutputStream = FileOutputStream(project.scene1File)
                        project.story?.let {
                            it.scenes?.let {
                                SceneDrawer.drawScene(context, it[0], 0, scaleWidth, scaleHeight)?.compress(Bitmap.CompressFormat.PNG,
                                        100, fileOutputStream)
                                fileOutputStream.flush()
                                fileOutputStream.close()
                            }
                        }
                        fileOutputStream = FileOutputStream(project.scene2File)
                        project.story?.let {
                            it.scenes?.let {
                                SceneDrawer.drawScene(context, it[1], 0, scaleWidth, scaleHeight)?.compress(Bitmap.CompressFormat.PNG,
                                        100, fileOutputStream)
                                fileOutputStream.flush()
                                fileOutputStream.close()
                            }
                        }
                    }
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
            it.onComplete()
        }
    }

    interface OnSaveActorImage {
        fun onSaveActorImage(filePath: String)
    }
}