package com.bomostory.sceneeditmodule.utils

import android.media.MediaPlayer
import com.bomostory.sceneeditmodule.basicdata.Music
import java.util.*

class MusicPlayerV2 {
    interface OnPlayChangedListener {
        fun OnPlayChanged(position: Int)
    }

    lateinit var music: Music

    val mediaPlayer = MediaPlayer()

    var timer: Timer? = null

    var isPause = false

    var playChangedListener: OnPlayChangedListener? = null

    fun init() {
        mediaPlayer.apply {
            if (!isPause) {
                reset()
                setDataSource(music.path)
                prepare()
            }
        }
    }

    fun start() {
        if (!isPause) {
            startTimer()
        }
        isPause = false
        mediaPlayer.start()
    }

    fun pause() {
        isPause = true
        mediaPlayer.pause()
    }

    fun seekTo(mSec: Int) {
        mediaPlayer.seekTo(mSec)
    }

    fun setVolume(volume: Float){
        mediaPlayer.setVolume(volume, volume)
    }

    fun stop() {
        isPause = false

        timer?.cancel()
        timer?.purge()
        timer = null

        mediaPlayer.stop()
    }

    fun release(){
        mediaPlayer.release()
    }

    private fun startTimer() {
        timer = Timer()
        timer?.scheduleAtFixedRate(object : TimerTask() {
            override fun run() {
                if (!isPause) {
                    playChangedListener?.OnPlayChanged(mediaPlayer.currentPosition)
                }
            }
        }, 0, 1)
    }

}